/*
 * thingimg.c dated 08/24/96 (translated 12/29/96)
 *
 * Author:
 * Thomas Binder
 * (binder@rbg.informatik.th-darmstadt.de)
 *
 * Idea:
 * Arno Welzel
 * (aw@inga.augusta.de)
 *
 * Purpose:
 * Very simple sample source for an external ThingImg-module. ThingImg
 * is the image loader for Thing. The external concept saves memory
 * (ThingImg only needs memory when loading an image) and allows much
 * more flexible image loading, because ThingImg may be replaced with
 * a different module (which, for example, loads JPEG instead of
 * (X)IMG), if necessary.
 *
 * ThingImg is a program _without any_ startup code, the program text
 * must begin with a function that expects its parameters on the stack
 * (see main in the Pure-C-source). This function will be called by
 * Thing and get some parameters. If magic does not contain the value
 * 0x27081969L, ThingImg has to quit immediately (this happens when
 * Thing wants to get rid of ThingImg).
 *
 * In all other cases (i.e. when magic has the correct value), the
 * parameter what tells ThingImg what to do:
 * TI_INIT - ThingImg gets the chance to prepare for its work. You may
 *           _not_ allocate permanent memory here! This is alwas true,
 *           furthermore, no AES-calls are allowed for ThingImg. What's
 *           allowed is the initialisation of structures, etc. The
 *           contents of the THINGIMG-structure, which the third
 *           parameter img_info points to, is not valid at this time
 *           (except the elements version, homepath, desk_w, desk_h, and
 *           confpath, which are always valid) and must not be read out
 *           or modified. The return value will be ignored.
 * TI_TEST - ThingImg has to test if a picture file has the correct
 *           format. The element filename points to the full path of
 *           the file. Moreover, the elements vdi_handle and use_palette
 *           are valid, the rest is of no importance. The result has
 *           to be non-zero in case the picture file is valid, zero
 *           otherwise.
 * TI_SIZE - ThingImg has to calculate the ammount of memory which is
 *           necessary to keep a picture file in a VDI-raster. See
 *           TI_TEST for details on which elements of the THINGIMG-
 *           structure are valid. The value ThingImg has to calculate
 *           is the number of Bytes necessary to keep the picture in
 *           a VDI-raster for the current color depth. This value has
 *           to be stored in the element pic_size of the THINGIMG-
 *           structure. The return value is zero when everything is OK,
 *           non-zero otherwise (e.g. a GEMDOS-errorcode).
 * TI_LOAD - ThingImg has to load a picture file and adjust it to the
 *           current color depth of the screen. The element filename of
 *           the THINGIMG-structure points to the path of the image,
 *           fd_addr of the MFDB in the element image points to a
 *           memory area which the final image has to be stored in
 *           (the rest of the MFDB is not pre-initialised!) The element
 *           use_palette tells whether the palette of the picutre (in
 *           case it has one) should be used. Vdi_handle contains the
 *           handle of a virtual VDI-workstation which ThingImg may
 *           use (it's not allowed to make any permanent changes to
 *           this workstation, except the color table).
 *           When the image has been loaded and adjusted, ThingImg
 *           has to save the current color table to the element
 *           old_palette (of course, this only applies if the image
 *           has its own palette and use_palette is non-zero). For that,
 *           only the entries between the first and the last influenced
 *           color index (inclusive) have to be copied, these two values
 *           have to put into the elements first and last. If there's
 *           no palette or the palette is irrelevant (e.g. if the video
 *           hardware supports direct color), first has to be left with
 *           its initial value -1!
 *           Now, the current color palette must be set in a way that
 *           the image will be displayed correctly (of course, only
 *           if necessary and with use_palette being non-zero). If the
 *           image is monochrome, the element is_mono has to be set
 *           to one, to zero otherwise.
 *           A return value of zero tells Thing that the image has
 *           been successfully loaded and adjusted. All other values
 *           indicate an error and that the image can't be used. In
 *           the latter case, the color palette must remain unchanged!
 *           If the palette of the image should be used, ThingImg
 *           should pay attention not to change the first 16 colors.
 *           If this can't be avoided, the changes should be the least
 *           visible, to allow the user to continue working without
 *           having to look at a distracting or even invisible desktop.
 *
 * The element version of the THINGIMG-structure always tells ThingImg,
 * which version the structure has. By the time of writing, this is
 * 0x102 (version 1.02). The version number may be used to detect
 * certain extensions of the structure, which always be downward
 * compatible.
 *
 * History:
 * < dropped in the English translation >
 */

#include <tos.h>
#include <string.h>
#include "thingimg.h"

/* NEVER FORGET: DO NOT LINK ANY STARTUPCODE!!! */

#define PW      320L
#define PH      200L
#define PICSIZE (+((PW + 15) / 16) * PH * 2)

long cdecl main(long magic, int what, THINGIMG *img_info)
{
 MFDB src;

/*
 * If magic does not contain the "magical" value, ThingImg has to
 * quit using Pterm() immediately. (Caution: No startupcode, so it's
 * not possible to use return for that!)
 */
 if (magic != TI_MAGIC)
  Pterm(0);
 switch (what)
 {
/* Initialisation, this example doesn't need one */
  case TI_INIT:
   return 0L;
/*
 * Test the image file. Since this example doesn't load files anyway,
 * the return value is always "OK". This also shows the possibility of
 * having a special module which does not load pictures, but, for
 * example, calculates a new mandelbrot set each time. (Note: I've
 * already written such a module, it's called ApfelImg and may be
 * found in the Thing 1.09-distribution.)
 */
  case TI_TEST:
   return 1L;
/*
 * The image's size in bytes is determined by the #defines. Caution:
 * The return value tells whether the calculation was successful, the
 * calculated size has to be stored in the THINGIMG-structure!
 */
  case TI_SIZE:
   img_info->pic_size = PICSIZE;
   return 0L;
/*
 * "Load" the image. This example simply generates a bitmap with a
 * fixed size and a "nice" stripe-pattern.
 */
  case TI_LOAD:
/* The final image has to be placed at the address given by the MFDB */
   memset(img_info->picture.fd_addr, 24, PICSIZE);
/* The rest of the MFDB has to be set up correctly! */
   img_info->picture.fd_w = (int)PW;
   img_info->picture.fd_h = (int)PH;
   img_info->picture.fd_wdwidth = (int)((PW + 15) / 16);
   img_info->picture.fd_stand = 0;
   img_info->picture.fd_nplanes = 1;
   src = img_info->picture;
   src.fd_stand = 1;
/*
 * Of course, the artificial bitmap has to be transformed into
 * device-dependent format, even though it's monochrome.
 */
   vr_trnfm(img_info->vdi_handle, &src, &img_info->picture);
/*
 * Tell Thing that the bitmap is monochrome. Of course, this may also
 * be found out be looking at the number of planes, but it's more
 * clearly this way.
 */
   img_info->is_mono = 1;
   return 0L;
/*
 * Should Thing support new opcodes for what in the future, which are
 * unknown to this ThingImg, simply return 0L in that case. Future
 * extensions to the interface will pay respect to that.
 */
  default:
   return 0L;
 }
}

/* EOF */

/*
 * thingimg.h dated 08/24/96 (structure version 1.02)
 * (translated 12/29/96)
 *
 * Authors:
 * Arno Welzel (aw@inga.augusta.de)
 * Thomas Binder (binder@rbg.informatik.th-darmstadt.de)
 *
 * Purpose:
 * Headerfile for the development of new image modules for Thing.
 *
 * History:
 * < dropped in the English translation >
 */

#include <vdi.h>

/* Constants */
#define TI_MAGIC    0x27081969L
#define TI_INIT     0
#define TI_TEST     1
#define TI_SIZE     2
#define TI_LOAD     3

/*
 * This structure will be passed to the external module, which has to
 * fill the elements correctly. See thingimg.c for details.
 */
typedef struct
{
/* version number of the structure, currently 0x102 */
    int     version;        /* Always valid */

/* VDI-handle for ThingImg */
    int     vdi_handle;     /* Valid for TI_TEST, TI_SIZE, and TI_LOAD */

/* Full path of the image file */
    char    *filename;      /* Valid for TI_TEST, TI_SIZE, and TI_LOAD */

/* Use the color palette (if any) of the image (1) or not (0)? */
    int     use_palette;    /* Valid for TI_TEST, TI_SIZE, and TI_LOAD */

/* Ammount of memory needed to keep the image in a VDI-raster (MFDB) */
    long    pic_size;       /* Valid for TI_SIZE and TI_LOAD */

/* MFDB of the image */
    MFDB    picture;        /* Valid for TI_LOAD */

/* Is the image monochrome (1) or not (0)? */
    int     is_mono;        /* Valid for TI_LOAD */

/* When use_palette is 1, the old palette has to be stored here */
    int     old_palette[256 * 3];   /* Valid for TI_LOAD */

/* First and last used index in old_palette */
    int     first;          /* Valid for TI_LOAD */
    int     last;           /* Valid for TI_LOAD */

/*
 * Homepath of ThingImg (useful for INF-files), only available when
 * version is >= 0x101, always with a backslash at the end!
 */
    char    *homepath;      /* Always valid */

/*
 * Size of the desktop's work-area in pixels, only available when
 * version is >= 0x102!
 */
    int     desk_w;         /* Always valid */
    int     desk_h;         /* Always valid */

/*
 * Access path of the configuration files of Thing (may be useful for
 * user-dependent INF-files). Always with a backslash at the end and
 * only available when version is >= 0x102. Caution: may be the same
 * as homepath!
 */
    char    *confpath;      /* Always valid */
} THINGIMG;

/* EOF */